@@ -250,110 +250,130 @@ GridScreen::initiateCommon (CompAction          *action,
 	      !optionGetCycleSizes ())) &&
 	    gw->lastTarget & where)
 	{
+	    /* TODO: 
+	     * instead of just using what was here
+	     * decide on how to deal with rounding/truncating/off by one, 
+	     * might be helpful in cases like using a windowed virtual machine
+	     * with a relatively prime number for width, 
+	     * examples 631, 797, 1021, 1279, 1439, 1913, 2039, 2557, 3833, ... NextPrime[{N}, -1] 
+	     */
+	    int slotWidth12  = workarea.width () / 8;
+	    int slotWidth17  = (workarea.width () / 6);
 	    int slotWidth25  = workarea.width () / 4;
 	    int slotWidth33  = (workarea.width () / 3) + cw->border ().left;
-	    int slotWidth17  = slotWidth33 - slotWidth25;
+	    int slotWidth50  = (workarea.width () / 2);
 	    int slotWidth66  = workarea.width () - slotWidth33;
 	    int slotWidth75  = workarea.width () - slotWidth25;
+	    int slotWidth100 = workarea.width ();
+	    int slotWidth37  = slotWidth75 / 2;
 
 	    if (props.numCellsX == 2) /* keys (1, 4, 7, 3, 6, 9) */
 	    {
-		if ((currentRect.width () == desiredRect.width () &&
-		     currentRect.x () == desiredRect.x ()) ||
-		    (gw->resizeCount < 1) || (gw->resizeCount > 5))
-		    gw->resizeCount = 3;
-
+		if ((currentRect.width () ==  desiredRect.width ()  &&
+		     currentRect.x ()     ==  desiredRect.x ())     ||
+		    (gw->resizeCount < 1) || (gw->resizeCount > 6))
+		{
+		    gw->resizeCount = 2;
+		}
 		/* tricky, have to allow for window constraints when
 		 * computing what the 33% and 66% offsets would be
 		 */
 		switch (gw->resizeCount)
 		{
 		    case 1:
-			desiredSlot.setWidth (slotWidth66);
-			desiredSlot.setX (workarea.x () +
-					  props.gravityRight * slotWidth33);
-			++gw->resizeCount;
-			break;
-
+		    desiredSlot.setWidth (slotWidth50);
+		    desiredSlot.setX (workarea.x () +  props.gravityRight * slotWidth50);
+		    ++gw->resizeCount;
+		    break;
+                
 		    case 2:
-			++gw->resizeCount;
-			break;
-
+		    desiredSlot.setWidth (slotWidth33);
+		    desiredSlot.setX (workarea.x () + props.gravityRight * slotWidth66);
+		    ++gw->resizeCount;
+		    break;
+                
 		    case 3:
-			desiredSlot.setWidth (slotWidth33);
-			desiredSlot.setX (workarea.x () +
-					  props.gravityRight * slotWidth66);
-			++gw->resizeCount;
-			break;
-
+		    desiredSlot.setWidth (slotWidth25);
+		    desiredSlot.setX (workarea.x () + props.gravityRight * slotWidth75);
+		    ++gw->resizeCount;
+		    break;
+                
 		    case 4:
-			desiredSlot.setWidth (slotWidth25);
-			desiredSlot.setX (workarea.x () +
-					  props.gravityRight * slotWidth75);
-			++gw->resizeCount;
-			break;
-
+		    desiredSlot.setWidth (slotWidth100);
+		    desiredSlot.setX (workarea.x ());
+		    ++gw->resizeCount;
+		    break;
+                
 		    case 5:
-			desiredSlot.setWidth (slotWidth75);
-			desiredSlot.setX (workarea.x () +
-					  props.gravityRight * slotWidth25);
-			++gw->resizeCount;
-			break;
-
+		    desiredSlot.setWidth (slotWidth75);
+		    desiredSlot.setX (workarea.x () + props.gravityRight * slotWidth25);
+		    ++gw->resizeCount;
+		    break;
+                
+		    case 6:
+		    desiredSlot.setWidth (slotWidth66);
+		    desiredSlot.setX (workarea.x () + props.gravityRight * slotWidth33);
+		    ++gw->resizeCount;
+		    break;
+                
 		    default:
-			break;
+		    break;
 		}
 	    }
 	    else /* keys (2, 5, 8) */
 	    {
-		if ((currentRect.width () == desiredRect.width () &&
-		     currentRect.x () == desiredRect.x ()) ||
-		    (gw->resizeCount < 1) || (gw->resizeCount > 5))
-		    gw->resizeCount = 1;
-
+		if ((currentRect.width () ==  desiredRect.width ()  &&
+		     currentRect.x ()     ==  desiredRect.x ())     ||
+		    (gw->resizeCount < 1) || (gw->resizeCount > 6))
+		{
+		    gw->resizeCount = 2;
+		}
 		switch (gw->resizeCount)
 		{
 		    case 1:
-			desiredSlot.setWidth (workarea.width () -
-					      (slotWidth17 * 2));
-			desiredSlot.setX (workarea.x () + slotWidth17);
-			++gw->resizeCount;
-			break;
-
+		    desiredSlot.setWidth (slotWidth100);
+		    desiredSlot.setX (workarea.x ());
+		    ++gw->resizeCount;
+		    break;
+		    
 		    case 2:
-			desiredSlot.setWidth ((slotWidth25 * 2) +
-					      (slotWidth17 * 2));
-			desiredSlot.setX (workarea.x () +
-					  (slotWidth25 - slotWidth17));
-			++gw->resizeCount;
-			break;
-
+		    desiredSlot.setWidth (slotWidth75);
+		    desiredSlot.setX (workarea.x () + slotWidth12);
+		    ++gw->resizeCount;
+		    break;
+                
 		    case 3:
-			desiredSlot.setWidth ((slotWidth25 * 2));
-			desiredSlot.setX (workarea.x () + slotWidth25);
-			++gw->resizeCount;
-			break;
-
+		    desiredSlot.setWidth (slotWidth66);
+		    desiredSlot.setX (workarea.x () + slotWidth17);
+		    ++gw->resizeCount;
+		    break;
+                
 		    case 4:
-			desiredSlot.setWidth (slotWidth33 -
-					      (cw->border ().left +
-					       cw->border ().right));
-			desiredSlot.setX (workarea.x () + slotWidth33);
-			++gw->resizeCount;
-			break;
-
+		    desiredSlot.setWidth ((slotWidth50));
+		    desiredSlot.setX (workarea.x () + slotWidth25);
+		    ++gw->resizeCount;
+		    break;
+                
 		    case 5:
-			++gw->resizeCount;
-			break;
-
+		    desiredSlot.setWidth (slotWidth33 - (cw->border ().left + cw->border ().right));
+		    desiredSlot.setX (workarea.x () + slotWidth33);
+		    ++gw->resizeCount;
+		    break;
+                
+		    case 6:
+		    desiredSlot.setWidth ((slotWidth25));
+		    desiredSlot.setX (workarea.x () + slotWidth37);
+		    ++gw->resizeCount;
+		    break;
+                
 		    default:
-			break;
+		    break;
 		}
 	    }
-
-	    if (gw->resizeCount == 6)
+	    if (gw->resizeCount > 6)
+	    {
 		gw->resizeCount = 1;
-
+	    }
 	    desiredRect = constrainSize (cw, desiredSlot);
 	}
 
@@ -488,10 +508,8 @@ GridScreen::glPaintRectangle (const GLScreenPaintAttrib &sAttrib,
     GLVertexBuffer  *streamingBuffer = GLVertexBuffer::streamingBuffer ();
     GLfloat         vertexData[12];
     GLushort        colorData[4];
-    unsigned short *color, *fillColor, *outlineColor;
-    unsigned short  averageFillColor[4], averageOutlineColor[4];
+    GLushort        *color;
     GLboolean       isBlendingEnabled;
-    bool            blend = !optionGetDisableBlend ();
 
     const float MaxUShortFloat = std::numeric_limits <unsigned short>::max ();
 
@@ -502,51 +520,16 @@ GridScreen::glPaintRectangle (const GLScreenPaintAttrib &sAttrib,
 
     sTransform.toScreenSpace (output, -DEFAULT_Z_CAMERA);
 
-    if (blend)
-    {
-	glGetBooleanv (GL_BLEND, &isBlendingEnabled);
-	glEnable (GL_BLEND);
-    }
-
-    fillColor = optionGetFillColor ();
-    outlineColor = optionGetOutlineColor ();
-
-    if (optionGetUseDesktopAverageColor ())
-    {
-	const unsigned short *averageColor = screen->averageColor ();
-
-	if (averageColor)
-	{
-	    memcpy (averageFillColor, averageColor, 4 * sizeof (unsigned short));
-	    averageFillColor[3] = MaxUShortFloat * 0.6;
-	    fillColor = averageFillColor;
-
-	    // Generate a lighter color based on border to create more contrast
-	    unsigned int averageColorLevel = (averageColor[0] + averageColor[1] + averageColor[2]) / 3;
-	    unsigned short *oc = averageOutlineColor;
-
-	    float colorMultiplier;
-	    if (averageColorLevel > MaxUShortFloat * 0.3)
-		colorMultiplier = 0.7; // make it darker
-	    else
-		colorMultiplier = 2.0; // make it lighter
-
-	    oc[3] = averageColor[3];
-	    oc[0] = MIN(MaxUShortFloat, ((float) averageColor[0]) * colorMultiplier) * oc[3] / MaxUShortFloat;
-	    oc[1] = MIN(MaxUShortFloat, ((float) averageColor[1]) * colorMultiplier) * oc[3] / MaxUShortFloat;
-	    oc[2] = MIN(MaxUShortFloat, ((float) averageColor[2]) * colorMultiplier) * oc[3] / MaxUShortFloat;
-
-	    outlineColor = averageOutlineColor;
-	}
-    }
+    glGetBooleanv (GL_BLEND, &isBlendingEnabled);
+    glEnable (GL_BLEND);
 
     for (iter = animations.begin (); iter != animations.end () && animating; ++iter)
     {
 	Animation& anim = *iter;
 
 	float curve = powf (CURVE_ANIMATION, -anim.progress);
-	float alpha = blend ? (fillColor[3] / MaxUShortFloat) * anim.opacity : 0.85;
-	color = fillColor;
+	float alpha = (optionGetFillColorAlpha () / MaxUShortFloat) * anim.opacity;
+	color = optionGetFillColor ();
 
 	colorData[0] = alpha * color[0];
 	colorData[1] = alpha * color[1];
@@ -583,8 +566,8 @@ GridScreen::glPaintRectangle (const GLScreenPaintAttrib &sAttrib,
 				      anim.currentRect.height () - 2);
 
 	/* draw outline */
-	alpha = blend ? (outlineColor[3] / MaxUShortFloat) * anim.opacity : 1;
-	color = outlineColor;
+	alpha = (optionGetOutlineColorAlpha () / MaxUShortFloat) * anim.opacity;
+	color = optionGetOutlineColor ();
 
 	colorData[0] = alpha * color[0];
 	colorData[1] = alpha * color[1];
@@ -616,8 +599,8 @@ GridScreen::glPaintRectangle (const GLScreenPaintAttrib &sAttrib,
     if (!animating)
     {
 	/* draw filled rectangle */
-	float alpha = blend ? fillColor[3] / MaxUShortFloat : 0.85;
-	color = fillColor;
+	float alpha = optionGetFillColorAlpha () / MaxUShortFloat;
+	color = optionGetFillColor ();
 
 	colorData[0] = alpha * color[0];
 	colorData[1] = alpha * color[1];
@@ -651,8 +634,8 @@ GridScreen::glPaintRectangle (const GLScreenPaintAttrib &sAttrib,
 			  rect.height () - 2);
 
 	/* draw outline */
-	alpha = outlineColor[3] / MaxUShortFloat;
-	color = outlineColor;
+	alpha = optionGetOutlineColorAlpha () / MaxUShortFloat;
+	color = optionGetOutlineColor ();
 
 	colorData[0] = alpha * color[0];
 	colorData[1] = alpha * color[1];
@@ -678,11 +661,8 @@ GridScreen::glPaintRectangle (const GLScreenPaintAttrib &sAttrib,
 	streamingBuffer->render (sTransform);
     }
 
-    if (blend)
-    {
-	if (!isBlendingEnabled)
-	    glDisable (GL_BLEND);
-    }
+    if (!isBlendingEnabled)
+	glDisable (GL_BLEND);
 }
 
 bool
@@ -898,7 +878,7 @@ GridScreen::handleEvent (XEvent *event)
 		    animations.at (current).fromRect = w->serverBorderRect ();
 		    animations.at (current).currentRect	= w->serverBorderRect ();
 		    animations.at (current).duration = optionGetAnimationDuration ();
-		    animations.at (current).progress = 0.0f;
+		    animations.at (current).timer = animations.at (current).duration;
 		    animations.at (current).targetRect = desiredSlot;
 		    animations.at (current).window = w->id();
 
@@ -1212,24 +1192,18 @@ GridScreen::preparePaint (int msSinceLastPaint)
     for (iter = animations.begin (); iter != animations.end (); ++iter)
     {
 	Animation& anim = *iter;
-	GLfloat msSinceLastPaintFloat = static_cast<GLfloat>(msSinceLastPaint);
-	GLfloat animDurationFloat = static_cast<GLfloat>(anim.duration);
-	GLfloat progress_delta = 1.0f;
+	anim.timer -= msSinceLastPaint;
 
-	if (animDurationFloat > 0.0f)
-	    progress_delta = msSinceLastPaintFloat / animDurationFloat;
+	if (anim.timer < 0)
+	    anim.timer = 0;
 
 	if (anim.fadingOut)
-	{
-	    anim.opacity -= progress_delta;
-	}
+	    anim.opacity -= msSinceLastPaint * 0.002;
 	else
-	{
 	    if (anim.opacity < 1.0f)
 		anim.opacity = anim.progress * anim.progress;
 	    else
 		anim.opacity = 1.0f;
-	}
 
 	if (anim.opacity < 0)
 	{
@@ -1238,10 +1212,10 @@ GridScreen::preparePaint (int msSinceLastPaint)
 	    anim.complete = true;
 	}
 
-	anim.progress = std::min<GLfloat>(anim.progress + progress_delta, 1.0);
+	anim.progress =	(anim.duration - anim.timer) / anim.duration;
     }
 
-    if (optionGetDrawStretchedWindow () && !optionGetDisableBlend ())
+    if (optionGetDrawStretchedWindow ())
     {
 	CompWindow *cw = screen->findWindow (CompOption::getIntOptionNamed (o, "window"));
 
@@ -1311,6 +1285,7 @@ Animation::Animation ()
     targetRect = CompRect (0, 0, 0, 0);
     currentRect = CompRect (0, 0, 0, 0);
     opacity = 0.0f;
+    timer = 0.0f;
     duration = 0;
     complete = false;
     fadingOut = false;
@@ -1418,7 +1393,7 @@ GridWindow::glPaint (const GLWindowPaintAttrib& attrib, const GLMatrix& matrix,
     {
 	Animation& anim = *iter;
 
-	if (anim.progress < 1.0f && anim.window == window->id())
+	if (anim.timer > 0.0f && anim.window == window->id())
 	{
 	    GLWindowPaintAttrib wAttrib(attrib);
 	    GLMatrix wTransform (matrix);
