@@ -24,6 +24,14 @@
  */
 
 #include "gtk-window-decorator.h"
+#include "gwd-settings.h"
+#include "gwd-theme-cairo.h"
+
+static void
+draw_window_decoration (decor_t *decor)
+{
+    gwd_theme_draw_window_decoration (gwd_theme, decor);
+}
 
 void
 destroy_normal_frame (decor_frame_t *frame)
@@ -37,26 +45,6 @@ destroy_bare_frame (decor_frame_t *frame)
     decor_frame_destroy (frame);
 }
 
-
-/*
- * get_titlebar_font
- *
- * Returns: PangoFontDescription * or NULL if using system font
- * Description: Helper function to get the font for the titlebar
- */
-static const PangoFontDescription *
-get_titlebar_font (decor_frame_t *frame)
-{
-    const gchar *titlebar_font;
-    g_object_get (settings, "titlebar-font", &titlebar_font, NULL);
-
-    /* Using system font */
-    if (!titlebar_font)
-	return NULL;
-    else
-	return frame->titlebar_font;
-}
-
 /*
  * frame_update_titlebar_font
  *
@@ -67,66 +55,31 @@ get_titlebar_font (decor_frame_t *frame)
 void
 frame_update_titlebar_font (decor_frame_t *frame)
 {
-    const PangoFontDescription *font_desc;
-    PangoFontDescription *free_font_desc;
-    PangoFontMetrics *metrics;
+    GtkWidget *style_window = gwd_theme_get_style_window (gwd_theme);
+    PangoFontDescription *font_desc = gwd_theme_get_titlebar_font (gwd_theme, frame);
     PangoLanguage *lang;
+    PangoFontMetrics *metrics;
+    gint ascent, descent;
 
-    free_font_desc = NULL;
     frame = gwd_decor_frame_ref (frame);
 
-    font_desc = get_titlebar_font (frame);
-    if (!font_desc)
-    {
-        GtkCssProvider *provider = gtk_css_provider_get_default ();
-        GtkStyleContext *context = gtk_style_context_new ();
-        GtkWidgetPath *path = gtk_widget_path_new ();
-
-        gtk_widget_path_prepend_type (path, GTK_TYPE_WIDGET);
-        gtk_style_context_set_path (context, path);
-        gtk_widget_path_free (path);
-
-        gtk_style_context_add_provider (context, GTK_STYLE_PROVIDER (provider), GTK_STYLE_PROVIDER_PRIORITY_FALLBACK);
-
-        gtk_style_context_save (context);
-        gtk_style_context_set_state (context, GTK_STATE_FLAG_NORMAL);
-        gtk_style_context_get (context, GTK_STATE_FLAG_NORMAL, "font", &free_font_desc, NULL);
-        gtk_style_context_restore (context);
-
-        font_desc = (const PangoFontDescription *) free_font_desc;
-    }
-
+    if (frame->pango_context == NULL)
+        frame->pango_context = gtk_widget_create_pango_context (style_window);
     pango_context_set_font_description (frame->pango_context, font_desc);
 
-    lang    = pango_context_get_language (frame->pango_context);
+    lang = pango_context_get_language (frame->pango_context);
     metrics = pango_context_get_metrics (frame->pango_context, font_desc, lang);
+    ascent = pango_font_metrics_get_ascent (metrics);
+    descent = pango_font_metrics_get_descent (metrics);
 
-    frame->text_height = PANGO_PIXELS (pango_font_metrics_get_ascent (metrics) +
-				pango_font_metrics_get_descent (metrics));
-
-    gwd_decor_frame_unref (frame);
+    frame->text_height = PANGO_PIXELS (ascent + descent);
 
     pango_font_metrics_unref (metrics);
+    pango_font_description_free (font_desc);
 
-    if (free_font_desc)
-        pango_font_description_free (free_font_desc);
+    gwd_decor_frame_unref (frame);
 }
 
-void
-update_frames_titlebar_fonts (gpointer key,
-			      gpointer value,
-			      gpointer user_data)
-{
-    frame_update_titlebar_font ((decor_frame_t *) value);
-}
-
-void
-update_titlebar_font ()
-{
-    gwd_frames_foreach (update_frames_titlebar_fonts, NULL);
-}
-
-
 /*
  * update_event_windows
  *
@@ -230,27 +183,18 @@ update_event_windows (WnckWindow *win)
 	    h = 0;
 
 	    if (actions & event_window_actions[i][j] && i >= k && i <= l)
-		(*theme_get_event_window_position) (d, i, j, width, height,
-						    &x, &y, &w, &h);
+		gwd_theme_get_event_window_position (gwd_theme, d, i, j, width, height,
+		                                     &x, &y, &w, &h);
 
-	    /* Reparenting mode - create boxes which we monitor motionnotify on */
-	    if (d->frame_window)
-	    {
-		BoxPtr box = &d->event_windows[i][j].pos;
-		box->x1  = x;
-		box->x2 = x + w;
-		box->y1 = y;
-		box->y2 = y + h;
-	    }
 	    /* Pixmap mode with window geometry - create small event windows */
-	    else if (!d->frame_window && w != 0 && h != 0)
+	    if (w != 0 && h != 0)
 	    {
 		XMapWindow (xdisplay, d->event_windows[i][j].window);
 		XMoveResizeWindow (xdisplay, d->event_windows[i][j].window,
 				   x, y, w, h);
 	    }
 	    /* No parent and no geometry - unmap all event windows */
-	    else if (!d->frame_window)
+	    else
 	    {
 		XUnmapWindow (xdisplay, d->event_windows[i][j].window);
 	    }
@@ -277,42 +221,23 @@ update_event_windows (WnckWindow *win)
 	    WNCK_WINDOW_ACTION_UNSTICK
 	};
 
-	/* Reparenting mode - if a box was set and we no longer need it reset its geometry */
-	if (d->frame_window &&
-	    button_actions[i] && !(actions & button_actions[i]))
-	{
-	    memset (&d->button_windows[i].pos, 0, sizeof (Box));
-	}
 	/* Pixmap mode - if a box was set and we no longer need it unmap its window */
-	else if (!d->frame_window &&
-		 button_actions[i] && !(actions & button_actions[i]))
+	if (button_actions[i] && !(actions & button_actions[i]))
 	{
 	    XUnmapWindow (xdisplay, d->button_windows[i].window);
 	    continue;
 	}
 
-	/* Reparenting mode - if there is a button position for this
-	 * button then set the geometry */
-	if (d->frame_window &&
-	    (*theme_get_button_position) (d, i, width, height, &x, &y, &w, &h))
-	{
-	    BoxPtr box = &d->button_windows[i].pos;
-	    box->x1 = x;
-	    box->y1 = y;
-	    box->x2 = x + w;
-	    box->y2 = y + h;
-	}
 	/* Pixmap mode - if there is a button position for this button then map the window
 	 * and resize it to this position */
-	else if (!d->frame_window &&
-		 (*theme_get_button_position) (d, i, width, height,
-					       &x, &y, &w, &h))
+	if (gwd_theme_get_button_position (gwd_theme, d, i, width, height,
+	                                   &x, &y, &w, &h))
 	{
 	    Window x11_win = d->button_windows[i].window;
 	    XMapWindow (xdisplay, x11_win);
 	    XMoveResizeWindow (xdisplay, x11_win, x, y, w, h);
 	}
-	else if (!d->frame_window)
+	else
 	{
 	    XUnmapWindow (xdisplay, d->button_windows[i].window);
 	}
@@ -355,8 +280,6 @@ max_window_name_width (WnckWindow *win)
     if (!d->layout)
     {
 	d->layout = pango_layout_new (d->frame->pango_context);
-	if (!d->layout)
-	    return 0;
 
 	pango_layout_set_wrap (d->layout, PANGO_WRAP_CHAR);
     }
@@ -407,7 +330,7 @@ update_window_decoration_name (WnckWindow *win)
 	gint w;
 
 	/* Cairo mode: w = SHRT_MAX */
-	if (theme_draw_window_decoration != draw_window_decoration)
+	if (!GWD_IS_THEME_CAIRO (gwd_theme))
 	{
 	    w = SHRT_MAX;
 	}
@@ -424,6 +347,10 @@ update_window_decoration_name (WnckWindow *win)
 		w = 1;
 	}
 
+        /* Ensure that a layout is created */
+        if (d->layout == NULL)
+            d->layout = pango_layout_new (d->frame->pango_context);
+
 	/* Set the maximum width for the layout (in case
 	 * decoration size < text width) since we
 	 * still need to show the buttons and the window name */
@@ -453,6 +380,31 @@ update_window_decoration_name (WnckWindow *win)
     }
 }
 
+static cairo_surface_t *
+surface_new_from_pixbuf (GdkPixbuf *pixbuf,
+                         GtkWidget *parent)
+{
+    guint width;
+    guint height;
+    cairo_surface_t *surface;
+    cairo_t *cr;
+
+    width  = gdk_pixbuf_get_width (pixbuf);
+    height = gdk_pixbuf_get_height (pixbuf);
+
+    surface = create_surface (width, height, parent);
+    if (!surface)
+        return NULL;
+
+    cr = cairo_create (surface);
+    gdk_cairo_set_source_pixbuf (cr, pixbuf, 0, 0);
+    cairo_set_operator (cr, CAIRO_OPERATOR_SOURCE);
+    cairo_paint (cr);
+    cairo_destroy (cr);
+
+    return surface;
+}
+
 /*
  * update_window_decoration_icon
  *
@@ -463,6 +415,7 @@ void
 update_window_decoration_icon (WnckWindow *win)
 {
     decor_t *d = g_object_get_data (G_OBJECT (win), "decor");
+    GtkWidget *style_window = gwd_theme_get_style_window (gwd_theme);
 
     /* Destroy old stuff */
     if (d->icon)
@@ -488,13 +441,8 @@ update_window_decoration_icon (WnckWindow *win)
 
 	g_object_ref (G_OBJECT (d->icon_pixbuf));
 
-	/* 32 bit pixmap on pixmap mode, 24 for reparenting */
-	if (d->frame_window)
-	    d->icon_surface = surface_new_from_pixbuf (d->icon_pixbuf,
-	                                               d->frame->style_window_rgb);
-	else
-	    d->icon_surface = surface_new_from_pixbuf (d->icon_pixbuf,
-	                                               d->frame->style_window_rgba);
+	d->icon_surface = surface_new_from_pixbuf (d->icon_pixbuf, style_window);
+
 	cr = cairo_create (d->icon_surface);
 	d->icon = cairo_pattern_create_for_surface (cairo_get_target (cr));
 	cairo_destroy (cr);
@@ -506,7 +454,6 @@ update_window_decoration_icon (WnckWindow *win)
  * Description: asks the rendering process to allow a size update
  * for pixmap synchronization */
 
-
 gboolean
 request_update_window_decoration_size (WnckWindow *win)
 {
@@ -530,7 +477,7 @@ request_update_window_decoration_size (WnckWindow *win)
 
     /* Ask the theme to tell us how much space it needs. If this is not successful
      * update the decoration name and return false */
-    if (!(*theme_calc_decoration_size) (d, w, h, name_width, &width, &height))
+    if (!gwd_theme_calc_decoration_size (gwd_theme, d, w, h, name_width, &width, &height))
     {
 	update_window_decoration_name (win);
 	return FALSE;
@@ -565,7 +512,7 @@ update_window_decoration_size (WnckWindow *win)
     cairo_surface_t   *surface, *buffer_surface = NULL;
     Picture           picture;
     Display           *xdisplay;
-    XRenderPictFormat *format;
+    GtkWidget         *style_window;
 
     if (win == NULL)
 	return FALSE;
@@ -576,20 +523,16 @@ update_window_decoration_size (WnckWindow *win)
 	return FALSE;
 
     xdisplay = GDK_DISPLAY_XDISPLAY (gdk_display_get_default ());
+    style_window = gwd_theme_get_style_window (gwd_theme);
 
     gdk_error_trap_push ();
 
-    /* Get the correct depth for the frame window in reparenting mode, otherwise
-     * enforce 32 */
-    if (d->frame_window)
-	surface = create_native_surface_and_wrap (d->width, d->height, d->frame->style_window_rgb);
-    else
-	surface = create_native_surface_and_wrap (d->width, d->height, d->frame->style_window_rgba);
+    surface = create_native_surface_and_wrap (d->width, d->height, style_window);
 
     gdk_flush ();
 
     /* Handle failure */
-    if (!surface || gdk_error_trap_pop ())
+    if (gdk_error_trap_pop () || !surface)
     {
 	if (surface)
 	    cairo_surface_destroy (surface);
@@ -598,17 +541,12 @@ update_window_decoration_size (WnckWindow *win)
 
     gdk_error_trap_push ();
 
-    if (d->frame_window)
-	buffer_surface = create_surface (d->width, d->height, d->frame->style_window_rgb);
-    else
-	buffer_surface = create_surface (d->width, d->height, d->frame->style_window_rgba);
+    buffer_surface = create_surface (d->width, d->height, style_window);
 
-    // FIXME: this should match the current monitor scaling!
-    cairo_surface_set_device_scale (buffer_surface, 1, 1);
     gdk_flush ();
 
     /* Handle failure */
-    if (!buffer_surface || gdk_error_trap_pop ())
+    if (gdk_error_trap_pop () || !buffer_surface)
     {
 	if (buffer_surface)
 	    cairo_surface_destroy (buffer_surface);
@@ -617,9 +555,8 @@ update_window_decoration_size (WnckWindow *win)
     }
 
     /* Create XRender context */
-    format = get_format_for_surface (d, buffer_surface);
     picture = XRenderCreatePicture (xdisplay, cairo_xlib_surface_get_drawable (buffer_surface),
-				    format, 0, NULL);
+                                    xformat_rgba, 0, NULL);
 
     /* Destroy the old pixmaps and pictures */
     if (d->surface)
@@ -707,7 +644,7 @@ draw_border_shape (Display	   *xdisplay,
     d.width   = width;
     d.height  = height;
     d.active  = TRUE;
-    d.draw    = theme_draw_window_decoration;
+    d.draw    = draw_window_decoration;
     d.picture = picture;
     d.context = c;
 
@@ -858,13 +795,13 @@ decor_frame_update_shadow (Display		  *xdisplay,
 						 1, 1,
 						 frame->win_extents.left,
 						 frame->win_extents.right,
-						 frame->win_extents.top + frame->titlebar_height,
+						 frame->win_extents.top,
 						 frame->win_extents.bottom,
 						 frame->win_extents.left -
 						 TRANSLUCENT_CORNER_SIZE,
 						 frame->win_extents.right -
 						 TRANSLUCENT_CORNER_SIZE,
-						 frame->win_extents.top + frame->titlebar_height -
+						 frame->win_extents.top -
 						 TRANSLUCENT_CORNER_SIZE,
 						 frame->win_extents.bottom -
 						 TRANSLUCENT_CORNER_SIZE,
@@ -889,12 +826,11 @@ decor_frame_update_shadow (Display		  *xdisplay,
 			     1, 1,
 			     frame->max_win_extents.left,
 			     frame->max_win_extents.right,
-			     frame->max_win_extents.top + frame->max_titlebar_height,
+			     frame->max_win_extents.top,
 			     frame->max_win_extents.bottom,
 			     frame->max_win_extents.left - TRANSLUCENT_CORNER_SIZE,
 			     frame->max_win_extents.right - TRANSLUCENT_CORNER_SIZE,
-			     frame->max_win_extents.top + frame->max_titlebar_height -
-			     TRANSLUCENT_CORNER_SIZE,
+			     frame->max_win_extents.top - TRANSLUCENT_CORNER_SIZE,
 			     frame->max_win_extents.bottom - TRANSLUCENT_CORNER_SIZE,
 			     opt_no_shadow,  /* No shadow when maximized */
 			     context_max,
@@ -961,8 +897,8 @@ update_frames_shadows (gpointer key,
     opts->active_shadow = &active_o;
     opts->inactive_shadow = &inactive_o;
 
-    (*theme_get_shadow) (frame, opts->active_shadow, TRUE);
-    (*theme_get_shadow) (frame, opts->inactive_shadow, FALSE);
+    gwd_theme_get_shadow (gwd_theme, frame, opts->active_shadow, TRUE);
+    gwd_theme_get_shadow (gwd_theme, frame, opts->inactive_shadow, FALSE);
 
     gwd_decor_frame_ref (frame);
 
@@ -989,34 +925,6 @@ update_frames_shadows (gpointer key,
 
 }
 
-static void
-get_shadow_common (decor_frame_t *d, decor_shadow_options_t *opts, gboolean active)
-{
-    decor_shadow_options_t *setting_opts = NULL;
-
-    if (active)
-	g_object_get (settings, "active-shadow", &setting_opts, NULL);
-    else
-	g_object_get (settings, "inactive-shadow", &setting_opts, NULL);
-
-    if (setting_opts)
-    {
-	memcpy (opts, setting_opts, sizeof (decor_shadow_options_t));
-    }
-}
-
-void
-cairo_get_shadow (decor_frame_t *frame, decor_shadow_options_t *opts, gboolean active)
-{
-    get_shadow_common (frame, opts, active);
-}
-
-void
-meta_get_shadow (decor_frame_t *frame, decor_shadow_options_t *opts, gboolean active)
-{
-    get_shadow_common (frame, opts, active);
-}
-
 int
 update_shadow (void)
 {
@@ -1266,21 +1174,18 @@ queue_decor_draw (decor_t *d)
 void
 update_default_decorations (GdkScreen *screen)
 {
+    GdkDisplay *display = gdk_display_get_default ();
+    Display *xdisplay = gdk_x11_display_get_xdisplay (display);
+    Window xroot = RootWindowOfScreen (gdk_x11_screen_get_xscreen (screen));
+    Atom bareAtom = XInternAtom (xdisplay, DECOR_BARE_ATOM_NAME, FALSE);
+    Atom activeAtom = XInternAtom (xdisplay, DECOR_ACTIVE_ATOM_NAME, FALSE);
+    GtkWidget *style_window = gwd_theme_get_style_window (gwd_theme);
     long	    *data;
-    Window	    xroot;
-    GdkDisplay	    *gdkdisplay = gdk_display_get_default ();
-    Display	    *xdisplay = gdk_x11_display_get_xdisplay (gdkdisplay);
-    Atom	    bareAtom, activeAtom;
     decor_frame_t   *frame;
     decor_frame_t   *bare_frame = gwd_get_decor_frame ("bare");
     decor_extents_t extents;
     unsigned int    i;
 
-    xroot = RootWindowOfScreen (gdk_x11_screen_get_xscreen (screen));
-
-    bareAtom   = XInternAtom (xdisplay, DECOR_BARE_ATOM_NAME, FALSE);
-    activeAtom = XInternAtom (xdisplay, DECOR_ACTIVE_ATOM_NAME, FALSE);
-
     if (bare_frame->border_shadow_active)
     {
 	decor_layout_t layout;
@@ -1358,7 +1263,7 @@ update_default_decorations (GdkScreen *screen)
 
         default_frames[i].d->context = i < WINDOW_TYPE_FRAMES_NUM ? &frame->window_context_active : &frame->window_context_inactive;
         default_frames[i].d->shadow  =  i < WINDOW_TYPE_FRAMES_NUM ? frame->border_shadow_active : frame->border_shadow_inactive;
-        default_frames[i].d->layout  = pango_layout_new (frame->pango_context);
+        default_frames[i].d->layout = NULL;
 
         decor_get_default_layout (default_frames[i].d->context, 1, 1, &default_frames[i].d->border_layout);
 
@@ -1368,12 +1273,10 @@ update_default_decorations (GdkScreen *screen)
         default_frames[i].d->frame = frame;
         default_frames[i].d->active = i < WINDOW_TYPE_FRAMES_NUM ? TRUE : FALSE;
 
-        extents.top += frame->titlebar_height;
-
-        default_frames[i].d->draw = theme_draw_window_decoration;
+        default_frames[i].d->draw = draw_window_decoration;
 	default_frames[i].d->surface = create_native_surface_and_wrap (default_frames[i].d->width,
 	                                                               default_frames[i].d->height,
-	                                                               frame->style_window_rgba);
+	                                                               style_window);
 
 	unsigned int j, k;
 
@@ -1460,10 +1363,8 @@ create_bare_frame (const gchar *type)
     frame->win_extents = _shadow_extents;
     frame->window_context_active = _shadow_context;
     frame->window_context_inactive = _shadow_context;
-    frame->window_context_no_shadow = _shadow_context;
     frame->max_window_context_active = _shadow_context;
     frame->max_window_context_inactive = _shadow_context;
-    frame->max_window_context_no_shadow = _shadow_context;
     frame->update_shadow = bare_frame_update_shadow;
 
     return frame;
@@ -1486,18 +1387,6 @@ create_normal_frame (const gchar *type)
         0, 0, 0, 0
     };
 
-    decor_context_t _window_context_no_shadow = {
-        { 0, 0, 0, 0 },
-        6, 6, 4, 6,
-        0, 0, 0, 0
-    };
-
-    decor_context_t _max_window_context_no_shadow = {
-        { 0, 0, 0, 0 },
-        6, 6, 4, 6,
-        0, 0, 0, 0
-    };
-
     decor_extents_t _win_extents         = { 6, 6, 6, 6 };
     decor_extents_t _max_win_extents     = { 6, 6, 4, 6 };
 
@@ -1506,10 +1395,8 @@ create_normal_frame (const gchar *type)
     frame->update_shadow = decor_frame_update_shadow;
     frame->window_context_active = _window_context;
     frame->window_context_inactive = _window_context;
-    frame->window_context_no_shadow = _window_context_no_shadow;
     frame->max_window_context_active = _max_window_context;
     frame->max_window_context_inactive = _max_window_context;
-    frame->max_window_context_no_shadow = _max_window_context_no_shadow;
 
     return frame;
 }
