@@ -25,6 +25,7 @@
 
 #ifndef _GTK_WINDOW_DECORATOR_H
 #define _GTK_WINDOW_DECORATOR_H
+
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
@@ -41,22 +42,12 @@
 #include <gdk/gdkx.h>
 #include <gdk/gdk.h>
 
-#ifdef USE_DBUS_GLIB
-#define DBUS_API_SUBJECT_TO_CHANGE
-#include <dbus/dbus.h>
-#include <dbus/dbus-glib-lowlevel.h>
-#endif
-
 #define WNCK_I_KNOW_THIS_IS_UNSTABLE
 #include <libwnck/libwnck.h>
 
 #include <cairo.h>
 #include <cairo-xlib.h>
 
-#if CAIRO_VERSION < CAIRO_VERSION_ENCODE(1, 1, 0)
-#define CAIRO_EXTEND_PAD CAIRO_EXTEND_NONE
-#endif
-
 #include <pango/pango-context.h>
 #include <pango/pangocairo.h>
 
@@ -72,11 +63,7 @@
 #define _(x)  gettext (x)
 #define N_(x) x
 
-#ifdef USE_METACITY
-#include <metacity-private/theme.h>
-#endif
-
-#include <gwd-fwd.h>
+#include "gwd-theme.h"
 
 extern const unsigned short ICON_SPACE;
 
@@ -103,17 +90,11 @@ unsigned long decorations;
 
 extern gboolean minimal;
 
-extern GWDSettingsNotified *notified;
-extern GWDSettings *settings;
-extern GWDSettingsWritable *writable;
+extern GWDTheme *gwd_theme;
 
 extern gdouble decoration_alpha;
-#ifdef USE_METACITY
-extern gboolean meta_button_layout_set;
-#endif
 
 extern Atom frame_input_window_atom;
-extern Atom frame_output_window_atom;
 extern Atom win_decor_atom;
 extern Atom win_blur_decor_atom;
 extern Atom wm_move_resize_atom;
@@ -170,7 +151,6 @@ typedef struct _decor_color {
     double b;
 } decor_color_t;
 
-
 #define IN_EVENT_WINDOW      (1 << 0)
 #define PRESSED_EVENT_WINDOW (1 << 1)
 
@@ -229,24 +209,15 @@ typedef void (*destroy_frame_proc) (decor_frame_t *);
 struct _decor_frame {
     decor_extents_t win_extents;
     decor_extents_t max_win_extents;
-    int		    titlebar_height;
-    int		    max_titlebar_height;
     decor_shadow_t *border_shadow_active;
     decor_shadow_t *border_shadow_inactive;
-    decor_shadow_t *border_no_shadow;
     decor_shadow_t *max_border_shadow_active;
     decor_shadow_t *max_border_shadow_inactive;
-    decor_shadow_t *max_border_no_shadow;
     decor_context_t window_context_active;
     decor_context_t window_context_inactive;
-    decor_context_t window_context_no_shadow;
     decor_context_t max_window_context_active;
     decor_context_t max_window_context_inactive;
-    decor_context_t max_window_context_no_shadow;
-    PangoFontDescription *titlebar_font;
     PangoContext	 *pango_context;
-    GtkWidget	         *style_window_rgba;
-    GtkWidget		 *style_window_rgb;
     gint		 text_height;
     gchar		 *type;
 
@@ -259,15 +230,10 @@ typedef struct _decor {
     decor_frame_t     *frame;
     event_window      event_windows[3][3];
     event_window      button_windows[BUTTON_NUM];
-    Box		      *last_pos_entered;
     guint	      button_states[BUTTON_NUM];
     Pixmap            x11Pixmap;
     cairo_surface_t   *surface;
     cairo_surface_t   *buffer_surface;
-    GdkWindow	      *frame_window;
-    GtkWidget         *decor_window;
-    GtkWidget	      *decor_event_box;
-    GtkWidget         *decor_image;
     cairo_t	      *cr;
     decor_layout_t    border_layout;
     decor_context_t   *context;
@@ -304,38 +270,6 @@ typedef struct _default_frame_references
 extern default_frame_references_t default_frames[WINDOW_TYPE_FRAMES_NUM * 2];
 const gchar * window_type_frames[WINDOW_TYPE_FRAMES_NUM];
 
-void     (*theme_get_shadow)		    (decor_frame_t *d,
-					     decor_shadow_options_t *,
-					     gboolean);
-
-void     (*theme_draw_window_decoration)    (decor_t *d);
-gboolean (*theme_calc_decoration_size)      (decor_t *d,
-					     int     client_width,
-					     int     client_height,
-					     int     text_width,
-					     int     *width,
-					     int     *height);
-void     (*theme_update_border_extents)     (decor_frame_t *frame);
-void     (*theme_get_event_window_position) (decor_t *d,
-
-					     gint    i,
-					     gint    j,
-					     gint    width,
-					     gint    height,
-					     gint    *x,
-					     gint    *y,
-					     gint    *w,
-					     gint    *h);
-gboolean (*theme_get_button_position)       (decor_t *d,
-					     gint    i,
-					     gint    width,
-					     gint    height,
-					     gint    *x,
-					     gint    *y,
-					     gint    *w,
-					     gint    *h);
-gfloat (*theme_get_title_scale)		    (decor_frame_t *frame);
-
 extern char *program_name;
 
 /* list of all decorations */
@@ -344,7 +278,6 @@ extern GHashTable    *frame_table;
 /* action menu */
 extern GtkWidget     *action_menu;
 extern gboolean      action_menu_mapped;
-extern decor_color_t _title_color[2];
 extern gint	     double_click_timeout;
 
 
@@ -363,7 +296,6 @@ extern GtkWidget  *switcher_label;
 extern decor_t    *switcher_window;
 
 extern XRenderPictFormat *xformat_rgba;
-extern XRenderPictFormat *xformat_rgb;
 
 /* frames.c */
 
@@ -452,9 +384,6 @@ update_event_windows (WnckWindow *win);
 int
 update_shadow (void);
 
-void
-update_titlebar_font ();
-
 void
 update_window_decoration (WnckWindow *win);
 
@@ -481,8 +410,7 @@ window_closed (WnckScreen *screen,
 
 void
 add_frame_window (WnckWindow *win,
-		  Window     frame,
-		  Bool	     mode);
+                  Window     frame);
 
 void
 remove_frame_window (WnckWindow *win);
@@ -506,14 +434,6 @@ decor_update_blur_property (decor_t *d,
 			    Region  right_region,
 			    int     right_offset);
 
-/* decorprops.c */
-
-void
-decor_update_window_property (decor_t *d);
-
-void
-decor_update_switcher_property (decor_t *d);
-
 /* cairo.c */
 
 #define CORNER_TOPLEFT     (1 << 0)
@@ -532,9 +452,6 @@ draw_shadow_background (decor_t		*d,
 			decor_shadow_t  *s,
 			decor_context_t *c);
 
-void
-draw_window_decoration (decor_t *d);
-
 void
 fill_rounded_rectangle (cairo_t       *cr,
 			double        x,
@@ -558,52 +475,8 @@ rounded_rectangle (cairo_t *cr,
 		   double  radius,
 		   int	   corner);
 
-gboolean
-calc_decoration_size (decor_t *d,
-		      gint    w,
-		      gint    h,
-		      gint    name_width,
-		      gint    *width,
-		      gint    *height);
-
-void
-update_border_extents ();
-
-gboolean
-get_button_position (decor_t *d,
-		     gint    i,
-		     gint    width,
-		     gint    height,
-		     gint    *x,
-		     gint    *y,
-		     gint    *w,
-		     gint    *h);
-
-void
-get_event_window_position (decor_t *d,
-			   gint    i,
-			   gint    j,
-			   gint    width,
-			   gint    height,
-			   gint    *x,
-			   gint    *y,
-			   gint    *w,
-			   gint    *h);
-
-gfloat
-get_title_scale (decor_frame_t *frame);
-
-void
-cairo_get_shadow (decor_frame_t *, decor_shadow_options_t *opts, gboolean active);
-
 /* gdk.c */
 
-GdkWindow *
-create_gdk_window (Window xframe);
-
-XRenderPictFormat *
-get_format_for_surface (decor_t *d, cairo_surface_t *surface);
-
 cairo_surface_t *
 create_surface (int	 w,
 	       int	 h,
@@ -614,62 +487,6 @@ create_native_surface_and_wrap (int	  w,
 			       int	  h,
 			       GtkWidget *parent_style_window);
 
-cairo_surface_t *
-surface_new_from_pixbuf (GdkPixbuf *pixbuf, GtkWidget *parent);
-
-/* metacity.c */
-
-#ifdef USE_METACITY
-
-MetaFrameType
-meta_get_frame_type_for_decor_type (const gchar *frame_type);
-
-void
-meta_draw_window_decoration (decor_t *d);
-
-gboolean
-meta_calc_decoration_size (decor_t *d,
-			   gint    w,
-			   gint    h,
-			   gint    name_width,
-			   gint    *width,
-			   gint    *height);
-
-gboolean
-meta_get_button_position (decor_t *d,
-			  gint    i,
-			  gint	  width,
-			  gint	  height,
-			  gint    *x,
-			  gint    *y,
-			  gint    *w,
-			  gint    *h);
-
-void
-meta_get_event_window_position (decor_t *d,
-				gint    i,
-				gint    j,
-				gint	width,
-				gint	height,
-				gint    *x,
-				gint    *y,
-				gint    *w,
-				gint    *h);
-
-gfloat
-meta_get_title_scale (decor_frame_t *);
-
-void
-meta_update_border_extents ();
-
-void
-meta_update_button_layout (const char *value);
-
-void
-meta_get_shadow (decor_frame_t *, decor_shadow_options_t *opts, gboolean active);
-
-#endif
-
 /* switcher.c */
 
 #define SWITCHER_ALPHA 0xa0a0
@@ -790,25 +607,6 @@ bottom_right_event (WnckWindow *win,
 		    decor_event *gtkwd_event,
 		    decor_event_type gtkwd_type);
 
-void
-frame_window_realized (GtkWidget *widget,
-		       gpointer  data);
-
-void
-frame_handle_button_press (GtkWidget      *widget,
-			   GdkEventButton *event,
-			   gpointer       user_data);
-
-void
-frame_handle_button_release (GtkWidget      *widget,
-			     GdkEventButton *event,
-			     gpointer       user_data);
-
-void
-frame_handle_motion (GtkWidget      *widget,
-		     GdkEventMotion *event,
-		     gpointer       user_data);
-
 GdkFilterReturn
 selection_event_filter_func (GdkXEvent *gdkxevent,
 			     GdkEvent  *event,
@@ -863,16 +661,7 @@ get_gtk_theme_variant (Window xwindow);
 /* settings.c */
 
 void
-set_frame_scale (decor_frame_t *frame,
-		 const gchar *font_str);
-
-void
-set_frames_scales (gpointer key,
-		   gpointer value,
-		   gpointer user_data);
-
-gboolean
-init_settings (GWDSettingsWritable *writable, WnckScreen *screen);
+init_settings (GWDSettings *settings);
 
 void
 fini_settings ();
